!-----------------------------------------------------------------------------
!
! Non-linear power spectrum of matter density fluctuations,
! P_dd(k), computed using the 3rd order perturbation theory.
!
! P_dd(k) = P_11(k) + P_22(k) + 2P_13(k)
!
! where P_11(k) is the linear power spectrum, supplied by the user.
! Ref. e.g., Eq.(11)-(14) of Jeong & Komatsu, ApJ, 651, 619 (2006)
!
! *** P(k) is in units of h^-3 Mpc^3, and k is in units of h Mpc^-1 ***
! (This is the same as CAMB definition.)
! 
! One can scale this result to an arbitrary redshift, z, by rescaling 
! the linear growth factor as
!
! P_dd(k,z) = [D(z)/D(z*)]^2 P_11(k) + [D(z)/D(z*)]^4 [P_22(k) + 2P_13(k)]
!
! where D(z)/D(z*) is the linear growth factor relative to the input linear
! power spectrum computed at z=z*.
!
! << sample code >>
!
! USE linearpk
! double precision :: k_ov_h,p11,p22,p13
! character(len=128) :: filename
! integer :: n
! filename='wmap5baosn_max_likelihood_matterpower.dat'
! n=896 ! # of lines in the file
! CALL open_linearpk(filename,n)
! k_ov_h=0.01d0 ! h Mpc^-1
! CALL pkd(k_ov_h,p11,p22,p13)
! print*,'P_11(k) at k=',k_ov_h,' h Mpc^-1 is',p11,' h^-3 Mpc^3'
! print*,'P_22(k) at k=',k_ov_h,' h Mpc^-1 is',p22,' h^-3 Mpc^3'
! print*,'P_13(k) at k=',k_ov_h,' h Mpc^-1 is',p13,' h^-3 Mpc^3'
! print*,'** Total power spectrum is P_11 + P_22 + 2*P_13. **'
! CALL close_linearpk
! end
!
! August 23, 2008
! E. Komatsu
!
!-----------------------------------------------------------------------------
SUBROUTINE PkD(ak,p11,p22,p13)
  IMPLICIT none
  DOUBLE PRECISION, parameter :: lnkmin = -9.2 ! kmin=1e-4 h Mpc^-1
  DOUBLE PRECISION, intent(OUT) :: ak,p11,p22,p13
  DOUBLE PRECISION :: ss1,ss2,ss3,ss4,eps,lnkmax,p13_d
  DOUBLE PRECISION :: dp13dq_d,dp22dq_d,linear_pk
  DOUBLE PRECISION, parameter :: pi=3.14159265358979d0
  EXTERNAL dp13dq_d,dp22dq_d
  !!$eps = ak*1.d-2
  eps = ak*0.17
  lnkmax = 3.d0
  !!$ lnkmax = 4.d0 ! made very little differnce. lnkmax=3 is good enough.
  if(dlog(eps)<lnkmin)then
     print*,'k is too small: must be greater than',exp(lnkmin)/0.17
     stop
  endif
!$ tree level PT
  p11 = linear_pk(ak)
!!$ P_d_d(k)
  !!$ one-loop PT: P_13(k)
  CALL qromb(dp13dq_d,lnkmin,lnkmax,ss1,ak)
  p13_d = p11*ss1/(2.*pi)**2.
  !!$ one-loop PT: P_22(k): use trick of Jain*Bertschinger(1994) (Eq.[22])
  CALL qromb1(dp22dq_d,lnkmin,dlog(eps),ss1,ak,0.d0)
  CALL qromb1(dp22dq_d,dlog(eps),dlog(ak-eps),ss2,ak,0.d0)
  CALL qromb1(dp22dq_d,dlog(ak-eps),dlog(ak+eps),ss3,ak,eps)
  CALL qromb1(dp22dq_d,dlog(ak+eps),lnkmax,ss4,ak,0.d0)
  p22 = 2.*(2.*ss1+ss2+ss3+ss4)/(2.*pi)**2.
  p13 = 0.5d0*p13_d
  return
END SUBROUTINE PkD
!------------------------------------------------
DOUBLE PRECISION FUNCTION dp13dq_d(lnq,k)
  IMPLICIT none
  DOUBLE PRECISION :: k,lnq,q
  DOUBLE PRECISION :: F3,linear_pk
  q = dexp(lnq)
  dp13dq_d = q*F3(k,q)*linear_pk(q)
  return
END FUNCTION dp13dq_d
!------------------------------------------------
DOUBLE PRECISION FUNCTION dp22dq_d(lnq,k,eps)
  IMPLICIT none
  DOUBLE PRECISION :: k,lnq,q,eps,lim
  DOUBLE PRECISION :: dFFdmu,ss,linear_pk
  EXTERNAL dFFdmu
  q = dexp(lnq)
  if (eps.eq.0.) then
     CALL qromb2(dFFdmu,-1.d0,1.d0,ss,q,k)
  else
     lim = (k**2+q**2-eps**2)/(2.*k*q)
     CALL qromb2(dFFdmu,-1.d0,lim,ss,q,k)
  endif
  dp22dq_d = q**3.*linear_pk(q)*ss
  return
END FUNCTION dp22dq_d
!------------------------------------------------
DOUBLE PRECISION FUNCTION dFFdmu(mu,q,k)
  IMPLICIT none
  DOUBLE PRECISION :: k,p,q,mu
  DOUBLE PRECISION :: F2,linear_pk
  p = sqrt(k**2.+q**2.-2.*k*q*mu)
  dFFdmu = F2(k,q,mu)**2.*linear_pk(p)
  return
END FUNCTION dFFdmu
!---------------------------------------
DOUBLE PRECISION FUNCTION F2(k,q,mu)
  IMPLICIT none
!!$ vec{p} = vec{k} - vec{q}
!!$ F2(vec{q},vec{k}-vec{q})
!!$    in Eq.(52) of Smith, Scoccimarro & Sheth (2006)
  DOUBLE PRECISION :: k,q,mu
  DOUBLE PRECISION :: p,kp,kq,pq
  kq = k*q*mu
  kp = k**2.-kq
  pq = kq-q**2 
  p  = sqrt(k**2+q**2-2*kq)
  F2 = 5./7. &
      + (1./2.)*pq/(p*q)*( (p/q)+(q/p) ) &
      + (2./7.)*pq**2./(p*q)**2.
  return
END FUNCTION F2
!---------------------------------------
DOUBLE PRECISION FUNCTION F3(k,q)
  IMPLICIT none
  DOUBLE PRECISION  k,q
!!$ Ref: Eq.[2.25] of Makino, Sasaki and Suto, PRD, 46, 585 (1992)
  if(q/k<1d2)then
     F3 = k**2./252.*( &
          12.*(k/q)**2 - 158. + 100.*(q/k)**2 - 42*(q/k)**4 &
          + (3./k**5/q**3)*(q**2-k**2)**3*(7.*q**2+2.*k**2.) &
            *dlog((k+q)/abs(k-q)))
  else
     F3 = k**2.*(-122./315.) ! asymptotic value for q/k->infinity.
                             ! accurate to 2e-5 at q/k=100
  endif
  return
END FUNCTION F3
